import os
import sys
import shutil
from glob import glob
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from contextlib import redirect_stdout

class ArgsContainer(object):

    def __init__(self):
        self.multiparser = Multiparser()
        self.helper = Helper()

    def _check_strain_length(self, strain_lens, flag):
        lengths = {}
        for m_l in strain_lens:
            if ":" not in m_l:
                print("Error: The assignment of {0} needs to contain "
                      "genome names and their length "
                      "of checked region!".format(flag))
                sys.exit()
            else:
                if m_l.split(":")[-1] == "all":
                    lengths[m_l.split(":")[0]] = m_l.split(":")[-1]
                else:
                    lengths[m_l.split(":")[0]] = int(m_l.split(":")[-1])
        return lengths

    def _check_track_name(self, lib, tracks, strand):
        if lib.split("/")[-1] in tracks["file"]:
            print("Error: Some filenames of the wiggle files are repeated!")
            sys.exit()
        else:
            tracks["file"].append(lib.split("/")[-1])
        with open(lib) as fh:
            for line in fh:
                line = line.strip()
                if line.startswith("track"):
                    track_name = line.split(" ")[-1]
                    if track_name in tracks["track"][strand]:
                        print("Error: Some names of the tracks in the "
                              "wiggle files are repeated!")
                        sys.exit()
                    else:
                        tracks["track"][strand].append(track_name) 

    def _create_working_wigs(self, out_folder, libs, wig_folder, tracks):
        new_libs = []
        if libs is not None:
            self.helper.check_make_folder(wig_folder)
            for lib in libs:
                self._check_track_name(lib.split(":")[0], tracks,
                                       lib.split(":")[-1])
                shutil.copy(lib.split(":")[0], wig_folder)
                wig = lib.split(":")[0].split("/")[-1]
                new_libs.append(":".join([wig, ":".join(lib.split(":")[1:])]))
        else:
            new_libs = None
        return new_libs

    def _check_replicates(self, replicates_tex, replicates_frag,
                          tex_lib, frag_lib):
        '''Check the replicate of frag and tex libs'''
        if (tex_lib is not None) and (replicates_tex is None):
            print("Error: No replicates numbers for "
                  "TEX treated libraries are assigned!")
            sys.exit()
        if (frag_lib is not None) and (replicates_frag is None):
            print("Error: No replicates numbers for "
                  "fragmented libraries are assigned!")
            sys.exit()
        if (replicates_tex is not None) and (replicates_frag is not None):
            replicates = {"tex": replicates_tex,
                          "frag": replicates_frag}
        elif replicates_tex is not None:
            replicates = {"tex": replicates_tex, "frag": -1}
        elif replicates_frag is not None:
            replicates = {"tex": -1, "frag": replicates_frag}
        else:
            print("Error: No replicates number was assigned!")
            sys.exit()
        if replicates["tex"] != -1:
            for rep in replicates["tex"]:
                if ("_" not in rep):
                    print("Error: Please check the input format of replicate_tex! "
                          "It should also contain condition name.")
                    sys.exit()
        if replicates["frag"] != -1:
            for rep in replicates["frag"]:
                if ("_" not in rep):
                    print("Error: Please check the input format of replicate_frag! "
                          "It should also contain condition name.")
                    sys.exit()
        return replicates

    def _check_assign_info(self, infos, file_type, wig_type):
        if file_type == "cond":
            index = 1
        elif file_type == "rep":
            index = 97
        for info, num in sorted(infos.items()):
            if (file_type == "cond") or (file_type == "rep"):
                if file_type == "cond":
                    if info != index:
                        print("Error: The condition number and order "
                              "of --tex_notex_libs should follow 1, 2, 3.")
                        sys.exit()
                elif file_type == "rep":
                    if ord(info) != index:
                        print("Error: The replicate index and order "
                              "of --tex_notex_libs should follow a, b, c.")
                        sys.exit()
                if (wig_type == "tex") and (num % 4 != 0):
                    print("Error: The --tex_notex_libs was assinged incorrectly. "
                          "Please check it again.")
                    sys.exit()
                elif (wig_type == "frag") and (num % 2 != 0):
                    print("Error: The --frag_libs was assinged incorrectly. "
                          "Please check it again.")
                    sys.exit()
                index += 1
            elif file_type == "strand":
                if wig_type == "frag":
                    if (infos["+"] != infos["-"]):
                        print("Error: The --frag_libs was assinged incorrectly. "
                              "Please check it again.")
                        sys.exit()
                if wig_type == "tex":
                    if (num % 2 != 0) and (infos["+"] != infos["-"]):
                        print("Error: The --tex_notex_libs was assinged incorrectly.  "
                              "Please check it again.")
                        sys.exit()


    def _check_tex_frag(self, libs, wig_type):
        conds = {}
        reps = {}
        strands = {}
        for lib in libs:
            datas = lib.split(":")
            if not datas[0].endswith(".wig"):
                print("Error: The input wiggle files should end with .wig!")
                sys.exit()
            if (datas[1] != "notex") and (
                    datas[1] != "tex") and (
                    datas[1] != "frag"):
                print("Error: Please assign \"tex\", \"notex\" or "
                      "\"frag\" to your input libraries.")
                sys.exit()
            try: 
                if int(datas[2]) not in conds.keys():
                    conds[int(datas[2])] = 0
                conds[int(datas[2])] += 1
            except ValueError:
                print("Error: Condition of libs should be assigned by integers!")
                sys.exit()
            if datas[3] not in reps.keys():
                reps[datas[3]] = 0
            reps[datas[3]] += 1
            if (datas[4] != "+") and (datas[4] != "-"):
                print("Error: Strand of libs should be assigned as + or -")
                sys.exit()
            if datas[4] not in strands.keys():
                strands[datas[4]] = 0
            strands[datas[4]] += 1
        self._check_assign_info(conds, "cond", wig_type)
        self._check_assign_info(reps, "rep", wig_type)
        self._check_assign_info(strands, "strand", wig_type)

    def _check_libs(self, tex_notex_libs, frag_libs):
        '''Check the libs of frag and tex'''
        if (tex_notex_libs is None) and (frag_libs is None):
            print("Error: No libraries assigned!!")
            sys.exit()
        elif (tex_notex_libs is not None) and (frag_libs is not None):
            libs = tex_notex_libs + frag_libs
            self._check_tex_frag(tex_notex_libs, "tex")
            self._check_tex_frag(frag_libs, "frag")
        elif (tex_notex_libs is not None):
            libs = tex_notex_libs
            self._check_tex_frag(tex_notex_libs, "tex")
        elif (frag_libs is not None):
            libs = frag_libs
            self._check_tex_frag(frag_libs, "frag")
        return libs

    def _check_condition_num(self, out_prefix, libs):
        high = 0
        for lib in libs:
            datas = lib.split(":")
            if int(datas[2]) > high:
                high = int(datas[2])
        if len(out_prefix) != high:
            print("Error: The number of --condition_names should be "
                  "the same to the condition of input libraries!")
            sys.exit()

    def _combine_files(self, ref_files, out_folder, filename):
        if ref_files is not None:
            tar_file = os.path.join(out_folder, filename)
            if os.path.exists(tar_file):
                os.remove(tar_file)
            for files in ref_files:
                for file_ in glob(files):
                    self.helper.merge_file(file_, tar_file)
            return tar_file
        else:
            return None

    def _merge_by_strain(self, wig_path, libs):
        strains = []
        merge_folder = os.path.join(wig_path, "merge_tmp")
        self.helper.check_make_folder(merge_folder)
        for wig in os.listdir(wig_path):
            if "_STRAIN_" in wig:
                strain = wig.split("_STRAIN_")[-1].replace(".wig", "")
                if strain not in strains:
                    strains.append(strain)
        for strain in strains:
            change_f = False
            change_r = False
            for wig in os.listdir(wig_path):
                filename = wig.split("_STRAIN_")
                if ("_STRAIN_" in wig) and (
                        filename[-1].replace(
                            ".wig", "") == strain):
                    for lib in libs:
                        if (filename[0] in lib) and (lib[-1] == "+"):
                            self.helper.merge_file(
                                os.path.join(wig_path, wig),
                                os.path.join(merge_folder,
                                             "tmp_forward.wig"))
                            change_f = True
                        elif (filename[0] in lib) and (lib[-1] == "-"):
                            self.helper.merge_file(
                                os.path.join(wig_path, wig),
                                os.path.join(merge_folder,
                                             "tmp_reverse.wig"))
                            change_r = True
            if change_f and change_r:
                change_f = False
                change_r = False
                shutil.move(os.path.join(merge_folder, "tmp_forward.wig"),
                            os.path.join(merge_folder,
                                         strain + "_forward.wig"))
                shutil.move(os.path.join(merge_folder, "tmp_reverse.wig"),
                            os.path.join(merge_folder,
                                         strain + "_reverse.wig"))
            else:
                print("Error: .wig files should be compose of "
                      "forward or reverse files.")
                sys.exit()
        self.helper.remove_all_content(wig_path, ".wig", "file")
        self.helper.move_all_content(merge_folder, wig_path, None)
        shutil.rmtree(merge_folder)

    def _parser_combine_wigs(self, subcommand):
        '''Check the wig folders of frag and tex, then merge them'''
        self.tex_path = None
        self.frag_path = None
        if subcommand == "transcript":
            if self.gffs is not None:
                self.multiparser.parser_gff(self.gffs, None)
                gff_path = self.gffs
        elif subcommand == "terminator":
            self.multiparser.parser_gff(self.gffs, None)
            gff_path = os.path.join(self.gffs, "tmp")
            tmp_file = os.path.join(self.out_folder, "tmp.txt")
            with open(tmp_file, 'w') as fh:
                with redirect_stdout(fh):
                    self.multiparser.parser_gff(gff_path, None)
            os.remove(tmp_file)
        else:
            self.multiparser.parser_gff(self.gffs, None)
            gff_path = self.gffs
        if self.tex_wigs is not None:
            self.tex_path = os.path.join(self.tex_wigs, "tmp")
            self.multiparser.parser_wig(self.tex_wigs)
            if self.gffs is not None:
                self.multiparser.combine_wig(gff_path, self.tex_path,
                                             None, self.libs)
            else:
                self._merge_by_strain(self.tex_path, self.libs)
            self.merge_wigs = self.tex_wigs
            self.wig_path = self.tex_path
        if self.frag_wigs is not None:
            self.frag_path = os.path.join(self.frag_wigs, "tmp")
            self.multiparser.parser_wig(self.frag_wigs)
            if self.gffs is not None:
                self.multiparser.combine_wig(gff_path, self.frag_path,
                                             None, self.libs)
            else:
                self._merge_by_strain(self.frag_path, self.libs)
            self.merge_wigs = self.frag_wigs
            self.wig_path = self.frag_path
        if (self.tex_path is not None) and (
                self.frag_path is not None):
            self = self._merge_wig()
        if (self.tex_path is None) and (
                self.frag_path is None):
            print("Error: There is no proper wig files assigned!!")
            sys.exit()
        return self

    def _merge_wig(self):
        '''Copy the wig files to one folder'''
        self.merge_wigs = os.path.join(self.out_folder, "merge_wigs")
        if (self.tex_wigs is not None) and (
                self.frag_wigs is not None):
            self.helper.check_make_folder(self.merge_wigs)
            self.wig_path = os.path.join(self.merge_wigs, "tmp")
            self.helper.check_make_folder(self.wig_path)
            for wig in os.listdir(self.tex_wigs):
                if os.path.isfile(os.path.join(self.tex_wigs, wig)):
                    shutil.copy(os.path.join(self.tex_wigs, wig),
                                self.merge_wigs)
            for wig in os.listdir(self.frag_wigs):
                if os.path.isfile(os.path.join(self.frag_wigs, wig)):
                    shutil.copy(os.path.join(self.frag_wigs, wig),
                                self.merge_wigs)
            for wig in os.listdir(self.tex_path):
                if os.path.isfile(os.path.join(self.tex_path, wig)):
                    shutil.copy(os.path.join(self.tex_path, wig),
                                self.wig_path)
            for wig in os.listdir(self.frag_path):
                if os.path.isfile(os.path.join(self.frag_path, wig)):
                    self.helper.merge_file(os.path.join(self.frag_path, wig),
                                           os.path.join(self.wig_path, wig))
        elif (self.tex_wigs is not None):
            self.merge_wigs = self.tex_wigs
        elif (self.frag_wigs is not None):
            self.merge_wigs = self.frag_wigs
        return self

    def _deal_multi_inputs(self, inputs, file_type, num, command):
        '''It is for split the input if it is assigned to multiple factors'''
        if inputs is not None:
            datas = inputs.split(",")
            if num is not None:
                if (len(datas) != num):
                    print("Error: the amount of {0} is not correct!!".format(
                        command))
            new_inputs = []
            for data in datas:
                if file_type == "float":
                    new_inputs.append(float(data.strip()))
                elif file_type == "int":
                    new_inputs.append(int(data.strip()))
                else:
                    new_inputs.append(data)
            return new_inputs
        else:
            return inputs

    def _gen_copy_new_folder(self, file_types, out_folder,
                             folder_name, ref_files, flag):
        if ref_files is not None:
            new_ref_folder = os.path.join(out_folder, folder_name)
            self.helper.check_make_folder(new_ref_folder)
            for files in ref_files:
                detect = False
                for file_ in glob(files):
                    for type_ in file_types:
                        if file_.endswith(type_):
                            detect = True
                if not detect:
                    print("Error: There some files don't exist in {0} or "
                          "the {0} is/are not ended with {1}!".format(
                          " and ".join(flag), " ".join(file_types)))
                    sys.exit()
                shutil.copy(file_, new_ref_folder)
            return new_ref_folder
        else:
            return None

    def container_ratt(self, ratt_path, element, transfer_type,
                       ref_embl, ref_gbk, target_fasta, ref_fasta, ratt_folder,
                       convert_to_gff_rnt_ptt, tar_annotation_folder,
                       compare_pair):
        self.ratt_path = ratt_path
        self.element = element
        self.transfer_type = transfer_type
        self.ref_embls = self._gen_copy_new_folder(
                [".embl"], ratt_folder, "temp_embl",
                ref_embl, ["--ref_embl_files"])
        self.ref_gbk = self._gen_copy_new_folder(
                [".gbk", ".gbff", ".gb"], ratt_folder, "temp_gbk",
                ref_gbk, ["--ref_gbk_files"])
        file_types = [".fa", ".fna", ".fasta"]
        self.tar_fastas = self._gen_copy_new_folder(
                file_types, ratt_folder, "temp_tar", target_fasta,
                ["--ref_fasta_files"])
        self.ref_fastas = self._gen_copy_new_folder(
                file_types, ratt_folder, "temp_ref", ref_fasta,
                ["--target_fasta_files"])
        self.output_path = ratt_folder
        self.convert = convert_to_gff_rnt_ptt
        self.gff_outfolder = tar_annotation_folder
        self.pairs = compare_pair
        return self

    def container_tsspredator(self, TSSpredator_path, compute_program,
                              fasta_files, annotation_files, lib,
                              output_prefix, height, height_reduction, factor,
                              factor_reduction, base_height, enrichment_factor,
                              processing_factor, replicate_match, out_folder,
                              validate_gene, merge_manual, strain_lengths,
                              compare_transcript_assembly, fuzzy, utr_length,
                              cluster, re_check_orphan, overlap_feature,
                              overlap_gff, remove_low_expression):
        if strain_lengths is not None:
            nt_lengths = self._check_strain_length(
                    strain_lengths, "--genome_lengths")
            self.strain_lengths = nt_lengths
        else:
            self.strain_lengths = strain_lengths
            if merge_manual is not None:
                self.strain_lengths = {"all": "all"}
        self.tsspredator_path = TSSpredator_path
        self.program = compute_program
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], out_folder, "tmp_fasta", fasta_files,
                ["--fasta_files"])
        self.gffs = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_anno", annotation_files,
                ["--annotation_files"])
        self.wig_folder = os.path.join(out_folder, "tmp_wig")
        self.helper.check_make_folder(self.wig_folder)
        tracks = {"file": [], "track": {"+": [], "-": []}}
        self.libs = self._create_working_wigs(out_folder, lib,
                                              self.wig_folder, tracks)
        self.libs = self._check_libs(self.libs, None)
        self._check_condition_num(output_prefix, self.libs)
        self.output_prefixs = output_prefix
        self.height = height
        self.height_reduction = height_reduction
        self.factor = factor
        self.factor_reduction = factor_reduction
        self.base_height = base_height
        self.enrichment_factor = enrichment_factor
        self.processing_factor = processing_factor
        self.repmatch = replicate_match
        self.out_folder = out_folder
        self.validate = validate_gene
        self.manual = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_manual", merge_manual,
                ["--manual_files_lengths"])
        self.ta_files = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_ta", compare_transcript_assembly,
                ["--compare_transcript_assembly"])
        self.fuzzy = fuzzy
        self.utr_length = utr_length
        self.cluster = cluster
        self.check_orphan = re_check_orphan
        self.overlap_feature = overlap_feature
        self.overlap_gffs = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_reference", overlap_gff,
                ["--compare_overlap_gff"])
        self.remove_low_expression = remove_low_expression
        return self

    def container_optimize(self, TSSpredator_path, fasta_file, annotation_file,
                           manual, out_folder, max_height,
                           max_height_reduction, max_factor,
                           max_factor_reduction, max_base_height,
                           max_enrichment_factor, max_processing_factor,
                           utr_length, lib, output_prefix, cluster, strain_lengths,
                           core, program, replicate_match, steps):
        self.tsspredator_path = TSSpredator_path
        if strain_lengths is not None:
            nt_lengths = self._check_strain_length(
                    strain_lengths, "--genome_lengths")
            self.strain_lengths = nt_lengths
        else:
            self.strain_lengths = strain_lengths
            if manual is not None:
                self.strain_lengths = {"all": "all"}
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], out_folder, "tmp_fasta",
                fasta_file, ["--fasta_files"])
        self.manuals = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_manual",
                manual, ["--manual_files"])
        self.gffs = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_anno",
                annotation_file, ["--annotation_files"])
        self.wigs = os.path.join(out_folder, "tmp_wig")
        self.helper.check_make_folder(self.wigs)
        tracks = {"file": [], "track": {"+": [], "-": []}}
        self.libs = self._create_working_wigs(out_folder, lib,
                                              self.wigs, tracks)
        self.libs = self._check_libs(self.libs, None)
        self.output_folder = out_folder
        self.height = max_height
        self.height_reduction = max_height_reduction
        self.factor = max_factor
        self.factor_reduction = max_factor_reduction
        self.base_height = max_base_height
        self.enrichment = max_enrichment_factor
        self.processing = max_processing_factor
        self.utr = utr_length
        self._check_condition_num(output_prefix, self.libs)
        self.replicate_name = output_prefix
        self.cluster = cluster
        self.cores = core
        self.program = program
        self.replicate = replicate_match
        self.steps = steps
        return self

    def _create_wig_folder(self, folder, libs):
        if libs is not None:
            self.helper.check_make_folder(folder)
            return folder
        else:
            return None

    def container_terminator(
            self, TransTermHP_path, expterm_path, RNAfold_path, out_folder,
            fasta_files, annotation_files, transcript_files, srna,
            decrease, highest_coverage, fuzzy_detect_coverage,
            fuzzy_within_transcript, fuzzy_downstream_transcript,
            fuzzy_within_gene, fuzzy_downstream_gene, transtermhp_folder,
            tex_notex_libs, frag_libs, tex_notex, replicates_tex,
            replicates_frag, table_best, min_loop_length, max_loop_length,
            min_stem_length, max_stem_length, min_AT_tail_length, miss_rate,
            range_u, keep_multi, window, shift):
        self.TransTermHP_path = TransTermHP_path
        self.expterm_path = expterm_path
        self.RNAfold_path = RNAfold_path
        self.out_folder = out_folder
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], out_folder, "tmp_fasta", fasta_files,
                ["--fasta_files"])
        self.gffs = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_anno", annotation_files,
                ["--annotation_files"])
        self.trans = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_ta", transcript_files,
                ["--transcript_files"])
        self.srnas = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_srna", srna, ["srna_files"])
        self.helper.check_make_folder(os.path.join(out_folder, "tmp_wig"))
        self.tex_wigs = self._create_wig_folder(
                os.path.join(out_folder, "tmp_wig", "tex_notex"),
                tex_notex_libs)
        self.frag_wigs = self._create_wig_folder(
                os.path.join(out_folder, "tmp_wig", "frag"), frag_libs)
        self.decrease = decrease
        self.cutoff_coverage = highest_coverage
        self.fuzzy = fuzzy_detect_coverage
        self.fuzzy_up_ta = fuzzy_within_transcript
        self.fuzzy_down_ta = fuzzy_downstream_transcript
        self.fuzzy_up_gene = fuzzy_within_gene
        self.fuzzy_down_gene = fuzzy_downstream_gene
        self.hp_folder = transtermhp_folder
        tracks = {"file": [], "track": {"+": [], "-": []}}
        self.tlibs = self._create_working_wigs(
                out_folder, tex_notex_libs, self.tex_wigs, tracks)
        self.flibs = self._create_working_wigs(
                out_folder, frag_libs, self.frag_wigs, tracks)
        self.libs = self._check_libs(self.tlibs, self.flibs)
        self.tex_notex = tex_notex
        self.replicates_tex = replicates_tex
        self.replicates_frag = replicates_frag
        self.replicates = self._check_replicates(
                replicates_tex, replicates_frag, tex_notex_libs, frag_libs)
        self.table_best = table_best
        self.min_loop = min_loop_length
        self.max_loop = max_loop_length
        self.min_stem = min_stem_length
        self.max_stem = max_stem_length
        self.at_tail = min_AT_tail_length
        self.miss_rate = miss_rate
        self.range_u = range_u
        self.keep_multi = keep_multi
        self.window = window
        self.shift = shift
        self = self._parser_combine_wigs("terminator")
        return self

    def container_transcript(self, tex_notex, modifys, length, annotation_files,
                             height, width, tolerance, tolerance_coverage,
                             replicates_tex, replicates_frag, out_folder,
                             tss_files, TSS_fuzzy, tex_treated_libs,
                             fragmented_libs, compare_feature_genome,
                             table_best, terminator_files, fuzzy_term, max_dist):
        if (compare_feature_genome is not None) and (annotation_files is None):
            print("Error: --annotation_files needs to be assigned if "
                  "--compare_feature_genome is assigned.")
            sys.exit()
        for modify in modifys:
            if (modify != "merge_overlap") and (
                    modify != "extend_5end") and (
                    modify != "extend_3end") and (
                    modify != "within_extend_ends") and (
                    modify != "none"):
                print("Error: --modify_transcript need to be assign as "
                      "\"merge_overlap\", \"extend_5end\", \"extend_3end\", "
                      "\"within_extend_ends\" or \"none\". "
                      "The assignment is wrong!")
                sys.exit()
        self.modify = modifys
        self.helper.check_make_folder(os.path.join(out_folder, "tmp_wig"))
        self.tex_wigs = self._create_wig_folder(
                os.path.join(out_folder, "tmp_wig", "tex_notex"),
                tex_treated_libs)
        self.frag_wigs = self._create_wig_folder(
                os.path.join(out_folder, "tmp_wig", "frag"), fragmented_libs)
        self.tex = tex_notex
        self.length = length
        self.gffs = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_anno", annotation_files,
                ["--annotation_files"])
        self.height = height
        self.width = width
        self.tolerance = tolerance
        self.low_cutoff = tolerance_coverage
        self.replicates_tex = replicates_tex
        self.replicates_frag = replicates_frag
        self.replicates = self._check_replicates(
                replicates_tex, replicates_frag,
                tex_treated_libs, fragmented_libs)
        self.out_folder = out_folder
        self.compare_tss = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_tss", tss_files, ["--tss_files"])
        self.fuzzy = TSS_fuzzy
        tracks = {"file": [], "track": {"+": [], "-": []}}
        self.tlibs = self._create_working_wigs(
                out_folder, tex_treated_libs, self.tex_wigs, tracks)
        self.flibs = self._create_working_wigs(
                out_folder, fragmented_libs, self.frag_wigs, tracks)
        self.libs = self._check_libs(self.tlibs, self.flibs)
        self.c_feature = compare_feature_genome
        self.table_best = table_best
        self.terms = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_term", terminator_files,
                ["--terminator_files"])
        self.fuzzy_term = fuzzy_term
        self.max_dist = max_dist
        self = self._parser_combine_wigs("transcript")
        return self

    def container_utr(self, tss_files, annotation_files,
                      transcript_assembly_files, terminator_files,
                      terminator_fuzzy, utr_folder, tss_source, base_5utr,
                      length, base_3utr, fuzzy_3utr, fuzzy_5utr):
        self.tsss = self._gen_copy_new_folder(
                [".gff"], utr_folder, "tmp_tss", tss_files, ["--tss_files"])
        self.gffs = self._gen_copy_new_folder(
                [".gff"], utr_folder, "tmp_anno", annotation_files,
                ["--annotation_files"])
        self.trans = self._gen_copy_new_folder(
                [".gff"], utr_folder, "tmp_ta", transcript_assembly_files,
                ["--transcript_files"])
        self.terms = self._gen_copy_new_folder(
                [".gff"], utr_folder, "tmp_term", terminator_files,
                ["--terminator_files"])
        self.fuzzy = terminator_fuzzy
        self.out_folder = utr_folder
        self.source = tss_source
        self.base_5utr = base_5utr
        self.base_3utr = base_3utr
        self.length = length
        self.fuzzy_3utr = fuzzy_3utr
        self.fuzzy_5utr = fuzzy_5utr
        return self

    def container_srna(self, rnafold, relplot_pl, mountain_pl, blastn, blastx,
                       blastdb, srna_folder, UTR_derived_sRNA,
                       annotation_files, TSS_files, transcript_files,
                       TSS_intergenic_fuzzy, TSS_5UTR_fuzzy, TSS_3UTR_fuzzy,
                       TSS_interCDS_fuzzy, import_info, processing_site_files,
                       fasta_files, mountain_plot, nr_format, srna_format,
                       sRNA_database_path, nr_database_path, cutoff_energy,
                       para_blast, run_intergenic_TEX_coverage,
                       run_intergenic_noTEX_coverage,
                       run_intergenic_fragmented_coverage, break_tran,
                       run_antisense_TEX_coverage,
                       run_antisense_noTEX_coverage,
                       run_antisense_fragmented_coverage, run_utr_TEX_coverage,
                       run_utr_noTEX_coverage, run_utr_fragmented_coverage,
                       max_length, min_length, tex_notex_libs, frag_libs,
                       replicates_tex, replicates_frag, tex_notex, blast_e_nr,
                       blast_e_srna, detect_sRNA_in_CDS, table_best,
                       decrease_intergenic, decrease_utr, fuzzy_intergenic,
                       fuzzy_utr, cutoff_nr_hit, sORF, overlap_percent_CDS,
                       terminator_files, terminator_fuzzy_in_sRNA,
                       terminator_fuzzy_out_sRNA, ignore_hypothetical_protein,
                       TSS_source, min_utr_coverage, promoter_tables,
                       ranking_promoter, promoter_name, compute_sec_str):
        self.rnafold = rnafold
        self.compute_sec_str = compute_sec_str
        self.para_blast = para_blast
        self.relplot_pl = relplot_pl
        self.mountain_pl = mountain_pl
        self.blastx = blastx
        self.blastn = blastn
        self.blastdb = blastdb
        self.out_folder = srna_folder
        self.utr_srna = UTR_derived_sRNA
        self.gffs = self._gen_copy_new_folder(
                [".gff"], srna_folder, "temp_anno", annotation_files,
                ["--annotation_files"])
        self.tss_folder = self._gen_copy_new_folder(
                [".gff"], srna_folder, "temp_tss", TSS_files, ["--tss_files"])
        self.trans = self._gen_copy_new_folder(
                [".gff"], srna_folder, "temp_ta", transcript_files,
                ["--transcript_files"])
        self.fuzzy_inter_tss = TSS_intergenic_fuzzy
        self.fuzzy_5utr_tss = TSS_5UTR_fuzzy
        self.fuzzy_3utr_tss = TSS_3UTR_fuzzy
        self.fuzzy_intercds_tss = TSS_interCDS_fuzzy
        self.fuzzy_tsss = {"5utr": self.fuzzy_5utr_tss,
                           "3utr": self.fuzzy_3utr_tss,
                           "interCDS": self.fuzzy_intercds_tss,
                           "inter": self.fuzzy_inter_tss}
        self.import_info = import_info
        self.helper.check_make_folder(os.path.join(srna_folder, "temp_wig"))
        self.tex_wigs = self._create_wig_folder(
                os.path.join(srna_folder, "temp_wig", "tex_notex"),
                tex_notex_libs)
        self.frag_wigs = self._create_wig_folder(
                os.path.join(srna_folder, "temp_wig", "frag"), frag_libs)
        self.pro_folder = self._gen_copy_new_folder(
                [".gff"], srna_folder, "temp_pro", processing_site_files,
                ["--processing_site_files"])
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], srna_folder,
                "temp_fasta", fasta_files, ["--fasta_files"])
        self.mountain = mountain_plot
        self.nr_format = nr_format
        self.srna_format = srna_format
        self.srna_database = sRNA_database_path
        self.nr_database = nr_database_path
        self.energy = cutoff_energy
        self.coverage_tex = self._deal_multi_inputs(
                run_intergenic_TEX_coverage, "float", 5,
                "--run_intergenic_TEX_coverage")
        self.coverage_notex = self._deal_multi_inputs(
                run_intergenic_noTEX_coverage, "float", 5,
                "--run_intergenic_noTEX_coverage")
        self.coverage_frag = self._deal_multi_inputs(
                run_intergenic_fragmented_coverage, "float", 5,
                "--run_intergenic_fragmented_coverage")
        self.anti_cover_tex = self._deal_multi_inputs(
                run_antisense_TEX_coverage, "float", 5,
                "--run_antisense_TEX_coverage")
        self.anti_cover_notex = self._deal_multi_inputs(
                run_antisense_noTEX_coverage, "float", 5,
                "--run_antisense_noTEX_coverage")
        self.anti_cover_frag = self._deal_multi_inputs(
                run_antisense_fragmented_coverage, "float", 5,
                "--run_antisense_fragmented_coverage")
        self.break_tran = self._deal_multi_inputs(
                break_tran, "float", 3,
                "--run_break_transcript")
        self.utr_tex_cover = self._deal_multi_inputs(
                run_utr_TEX_coverage, "str", 3, "--run_utr_TEX_coverage")
        self.utr_notex_cover = self._deal_multi_inputs(
                run_utr_noTEX_coverage, "str", 3, "--run_utr_TEX_coverage")
        self.utr_frag_cover = self._deal_multi_inputs(
                run_utr_fragmented_coverage, "str", 3,
                "--run_utr_fragmented_coverage")
        self.max_len = max_length
        self.min_len = min_length
        tracks = {"file": [], "track": {"+": [], "-": []}}
        self.tlibs = self._create_working_wigs(
                srna_folder, tex_notex_libs, self.tex_wigs, tracks)
        self.flibs = self._create_working_wigs(
                srna_folder, frag_libs, self.frag_wigs, tracks)
        self.libs = self._check_libs(self.tlibs, self.flibs)
        self.replicates_tex = replicates_tex
        self.replicates_frag = replicates_frag
        self.replicates = self._check_replicates(
                replicates_tex, replicates_frag, tex_notex_libs, frag_libs)
        self.tex_notex = tex_notex
        self.e_nr = blast_e_nr
        self.e_srna = blast_e_srna
        self.in_cds = detect_sRNA_in_CDS
        self.table_best = table_best
        self.decrease_inter = decrease_intergenic
        self.decrease_utr = decrease_utr
        self.fuzzy_inter = fuzzy_intergenic
        self.fuzzy_utr = fuzzy_utr
        self.nr_hits_num = cutoff_nr_hit
        self.sorf_file = self._gen_copy_new_folder(
                [".gff"], srna_folder, "temp_sorf", sORF, ["--sorf_files"])
        self.cutoff_overlap = overlap_percent_CDS
        self.terms = self._gen_copy_new_folder(
                [".gff"], srna_folder, "temp_term", terminator_files,
                ["--terminator_files"])
        self.fuzzy_b = terminator_fuzzy_in_sRNA
        self.fuzzy_a = terminator_fuzzy_out_sRNA
        self.hypo = ignore_hypothetical_protein
        self.tss_source = TSS_source
        self.min_utr = min_utr_coverage
        self.promoter_table = self._combine_files(
                promoter_tables, srna_folder, "tmp_promoter_table")
        if ranking_promoter < 1:
            print("Error: --ranking_time_promoter must larger than 1...")
            sys.exit()
        self.rank_promoter = ranking_promoter
        self.promoter_name = promoter_name
        self = self._parser_combine_wigs("srna")
        return self

    def container_intersrna(self, file_type, files, args_srna, prefix,
                            gff_file, tran_file, tss_file, pro_file, fuzzy):
        '''Especially for intergenic and antisense sRNA'''
        args_srna.file_type = file_type
        args_srna.gff_file = gff_file
        args_srna.tran_file = tran_file
        args_srna.tss_file = tss_file
        args_srna.pro_file = pro_file
        args_srna.fuzzy = fuzzy
        args_srna.prefix = prefix
        if file_type == "frag":
            args_srna.wig_f_file = os.path.join(
                    args_srna.frag_path, "_".join([prefix, "forward.wig"]))
            args_srna.wig_r_file = os.path.join(
                    args_srna.frag_path, "_".join([prefix, "reverse.wig"]))
            args_srna.wig_folder = args_srna.frag_wigs
            args_srna.input_libs = args_srna.flibs
            args_srna.output_file = files["frag_gff"]
            args_srna.output_table = files["frag_csv"]
            args_srna.cutoffs = args_srna.coverage_frag
            args_srna.tss_source = True
            args_srna.cut_notex = args_srna.coverage_frag
            args_srna.anti_notex_cutoff = None
        else:
            args_srna.wig_f_file = os.path.join(
                    args_srna.tex_path, "_".join([prefix, "forward.wig"]))
            args_srna.wig_r_file = os.path.join(
                    args_srna.tex_path, "_".join([prefix, "reverse.wig"]))
            args_srna.wig_folder = args_srna.tex_wigs
            args_srna.input_libs = args_srna.tlibs
            args_srna.output_file = files["tex_gff"]
            args_srna.output_table = files["tex_csv"]
            args_srna.cutoffs = args_srna.coverage_tex
            args_srna.tss_source = args_srna.tss_source
            args_srna.cut_notex = args_srna.coverage_notex
            args_srna.anti_notex_cutoff = args_srna.anti_cover_notex
        return args_srna

    def container_utrsrna(self, gff, tran, tss, files, pro, fasta, file_type,
                          prefix, args_srna):
        '''Especially for UTR-derived sRNA'''
        args_srna.file_type = file_type
        args_srna.gff_file = gff
        args_srna.ta_file = tran
        args_srna.tss_file = tss
        args_srna.pro_file = pro
        args_srna.prefix = prefix
        args_srna.seq_file = fasta
        if file_type == "frag":
            args_srna.wig_f_file = os.path.join(
                    args_srna.frag_path, "_".join([prefix, "forward.wig"]))
            args_srna.wig_r_file = os.path.join(
                    args_srna.frag_path, "_".join([prefix, "reverse.wig"]))
            args_srna.wig_folder = args_srna.frag_wigs
            args_srna.input_libs = args_srna.flibs
            args_srna.output_file = files["frag_gff"]
            args_srna.output_table = files["frag_csv"]
            args_srna.utr_coverages = args_srna.utr_frag_cover
            args_srna.notex = None
        else:
            args_srna.wig_f_file = os.path.join(
                    args_srna.tex_path, "_".join([prefix, "forward.wig"]))
            args_srna.wig_r_file = os.path.join(
                    args_srna.tex_path, "_".join([prefix, "reverse.wig"]))
            args_srna.wig_folder = args_srna.tex_wigs
            args_srna.input_libs = args_srna.tlibs
            args_srna.output_file = files["tex_gff"]
            args_srna.output_table = files["tex_csv"]
            args_srna.utr_coverages = args_srna.utr_tex_cover
            args_srna.notex = args_srna.utr_notex_cover
        args_srna.coverages = {"5utr": args_srna.utr_coverages[0],
                               "3utr": args_srna.utr_coverages[1],
                               "interCDS": args_srna.utr_coverages[2]}
        if args_srna.notex is not None:
            args_srna.cover_notex = {"5utr": args_srna.notex[0],
                                     "3utr": args_srna.notex[1],
                                     "interCDS": args_srna.notex[2]}
        else:
            args_srna.cover_notex = None
        return args_srna

    def extend_inter_container(self, args_srna, tsss, pros,
                               nums, output, out_table, texs, detects,
                               cutoff_coverage, notex):
        '''Especially for intergenic and antisense sRNA'''
        args_srna.tsss = tsss
        args_srna.pros = pros
        args_srna.nums = nums
        args_srna.output = output
        args_srna.out_table = out_table
        args_srna.texs = texs
        args_srna.detects = detects
        args_srna.cutoff_coverage = cutoff_coverage
        args_srna.notex = notex
        return args_srna

    def extend_utr_container(self, args_srna, cdss, tsss, pros,
                             out, out_t, texs):
        '''Especially for UTR-derived sRNA'''
        args_srna.cdss = cdss
        args_srna.tsss = tsss
        args_srna.pros = pros
        args_srna.out = out
        args_srna.out_t = out_t
        args_srna.texs = texs
        args_srna.utrs = []
        args_srna.srnas = []
        return args_srna

    def container_sorf(self, sorf_folder, UTR_derived_sORF, transcript_files,
                       annotation_files, TSS_files, utr_length, min_length,
                       max_length, cutoff_intergenic_coverage,
                       cutoff_antisense_coverage, cutoff_5utr_coverage,
                       cutoff_3utr_coverage, cutoff_interCDS_coverage,
                       fasta_files, tex_notex_libs, frag_libs, tex_notex,
                       replicates_tex, replicates_frag, table_best,
                       sRNA_files, start_codon, stop_codon, cutoff_background,
                       fuzzy_rbs, rbs_not_after_TSS, print_all_combination,
                       best_no_sRNA, best_no_TSS, ignore_hypothetical_protein,
                       min_rbs_distance, max_rbs_distance):
        self.out_folder = sorf_folder
        self.utr_detect = UTR_derived_sORF
        self.trans = self._gen_copy_new_folder(
                [".gff"], sorf_folder, "temp_ta", transcript_files,
                ["--transcript_files"])
        self.gffs = self._gen_copy_new_folder(
                [".gff"], sorf_folder, "temp_anno", annotation_files,
                ["--annotation_files"])
        self.tsss = self._gen_copy_new_folder(
                [".gff"], sorf_folder, "temp_tss", TSS_files, ["--tss_files"])
        self.utr_length = utr_length
        self.min_len = min_length
        self.max_len = max_length
        self.helper.check_make_folder(os.path.join(sorf_folder, "temp_wig"))
        self.tex_wigs = self._create_wig_folder(
                os.path.join(sorf_folder, "temp_wig", "tex_notex"),
                tex_notex_libs)
        self.frag_wigs = self._create_wig_folder(
                os.path.join(sorf_folder, "temp_wig", "frag"), frag_libs)
        self.cutoff_inter = cutoff_intergenic_coverage
        self.cutoff_anti = cutoff_antisense_coverage
        self.cutoff_5utr = cutoff_5utr_coverage
        self.cutoff_3utr = cutoff_3utr_coverage
        self.cutoff_intercds = cutoff_interCDS_coverage
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], sorf_folder,
                "temp_fasta", fasta_files, ["--fasta_files"])
        tracks = {"file": [], "track": {"+": [], "-": []}}
        self.tlibs = self._create_working_wigs(
                sorf_folder, tex_notex_libs, self.tex_wigs, tracks)
        self.flibs = self._create_working_wigs(
                sorf_folder, frag_libs, self.frag_wigs, tracks)
        self.libs = self._check_libs(self.tlibs, self.flibs)
        self.tex_notex = tex_notex
        self.replicates_tex = replicates_tex
        self.replicates_frag = replicates_frag
        self.replicates = self._check_replicates(
                replicates_tex, replicates_frag, tex_notex_libs, frag_libs)
        self.table_best = table_best
        self.srnas = self._gen_copy_new_folder(
                [".gff"], sorf_folder, "temp_srna", sRNA_files,
                ["--srna_files"])
        self.start_codon = start_codon
        self.stop_codon = stop_codon
        self.background = cutoff_background
        self.fuzzy_rbs = fuzzy_rbs
        self.noafter_tss = rbs_not_after_TSS
        self.print_all = print_all_combination
        self.no_srna = best_no_sRNA
        self.no_tss = best_no_TSS
        self.hypo = ignore_hypothetical_protein
        self.min_rbs = min_rbs_distance
        self.max_rbs = max_rbs_distance
        self = self._parser_combine_wigs("sorf")
        return self

    def container_srna_target(
            self, rnaplfold_path, rnaplex_path, rnaup_path, annotation_files,
            fasta_files, sRNA_files, query_sRNA, program, interaction_length,
            window_size_target, span_target, window_size_srna, span_srna,
            unstructured_region_RNAplex_target,
            unstructured_region_RNAplex_srna, unstructured_region_RNAup,
            energy_threshold, duplex_distance, top, starget_output_folder,
            process_rnaplex, process_rnaup, continue_rnaup,
            potential_target_start, potential_target_end, target_feature):
        self.rnaplfold_path = rnaplfold_path
        self.rnaplex_path = rnaplex_path
        self.rnaup_path = rnaup_path
        self.gffs = self._gen_copy_new_folder(
                [".gff"], starget_output_folder, "tmp_anno",
                annotation_files, ["--annotation_files"])
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], starget_output_folder,
                "tmp_fasta", fasta_files, ["--fasta_files"])
        self.srnas = self._gen_copy_new_folder(
                [".gff"], starget_output_folder, "tmp_srna", sRNA_files,
                ["--srna_files"])
        self.query = query_sRNA
        self.program = program
        self.inter_length = interaction_length
        self.win_size_t = window_size_target
        self.span_t = span_target
        self.win_size_s = window_size_srna
        self.span_s = span_srna
        self.unstr_region_rnaplex_t = unstructured_region_RNAplex_target
        self.unstr_region_rnaplex_s = unstructured_region_RNAplex_srna
        self.unstr_region_rnaup = unstructured_region_RNAup
        self.energy = energy_threshold
        self.duplex_dist = duplex_distance
        self.top = top
        self.out_folder = starget_output_folder
        self.core_plex = process_rnaplex
        self.core_up = process_rnaup
        self.continue_rnaup = continue_rnaup
        self.tar_start = potential_target_start
        self.tar_end = potential_target_end
        self.features = target_feature
        return self

    def container_goterm(self, annotation_files, goterm_output_folder,
                         UniProt_id, go_obo, goslim_obo, transcript_files):
        self.gffs = self._gen_copy_new_folder(
                [".gff"], goterm_output_folder, "tmp_anno", annotation_files,
                ["--annotation_files"])
        self.out_folder = goterm_output_folder
        self.uniprot = UniProt_id
        self.go = go_obo
        self.goslim = goslim_obo
        self.trans = self._gen_copy_new_folder(
                [".gff"], goterm_output_folder, "tmp_ta", transcript_files,
                ["--transcript_files"])
        return self

    def container_sublocal(self, Psortb_path, annotation_files, fasta_files,
                           bacteria_type, difference_multi,
                           sublocal_output_folder, transcript_files):
        self.psortb_path = Psortb_path
        self.gffs = self._gen_copy_new_folder(
                [".gff"], sublocal_output_folder, "tmp_anno",
                annotation_files, ["--annotation_files"])
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], sublocal_output_folder,
                "tmp_fa", fasta_files, ["--fasta_files"])
        self.gram = bacteria_type
        self.fuzzy = difference_multi
        self.out_folder = sublocal_output_folder
        self.trans = self._gen_copy_new_folder(
                [".gff"], sublocal_output_folder, "tmp_ta",
                transcript_files, ["--transcript_files"])
        return self

    def container_ppi(self, annotation_files, proteinID_strains,
                      without_strain_pubmed, species_STRING, score,
                      ppi_output_folder, node_size, query):
        self.ptts = self._gen_copy_new_folder(
                [".gff"], ppi_output_folder, "temp_anno",
                annotation_files, ["--annotation_files"])
        self.strains = proteinID_strains
        self.no_specific = without_strain_pubmed
        self.species = species_STRING
        self.score = score
        self.out_folder = ppi_output_folder
        self.size = node_size
        self.querys = query
        return self

    def container_promoter(self, MEME_path, GLAM2_path, out_folder, tex_libs,
                           TSS_files, fasta_files, num_motif, nt_before_TSS,
                           motif_width, TSS_source, annotation_files, end_run,
                           combine_all, e_value, para, program):
        self.meme_path = MEME_path
        self.glam2_path = GLAM2_path
        self.program = program
        self.end_run = end_run
        if (program.lower() != "both") and (
                program.lower() != "meme") and (
                program.lower() != "glam2"):
            print("Error: Please assign meme or glam2 or both to --program.")
            sys.exit()
        self.output_folder = out_folder
        self.tsss = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_tss", TSS_files, ["--tss_files"])
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], out_folder, "tmp_fasta",
                fasta_files, ["--fasta_files"])
        self.num_motif = num_motif
        self.nt_before = nt_before_TSS
        self.widths = motif_width
        self.source = TSS_source
        self.tex_wigs = None
        self.frag_wigs = None
        self.gffs = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_anno", annotation_files,
                ["--annotation_files"])
        self.combine = combine_all
        self.e_value = e_value
        self.para = para
        if not TSS_source:
            if tex_libs is None:
                print("Error: if --tss_source is False, please assign "
                      "--tex_libs and --annotation_files as well!")
                sys.exit()
            else:
                if annotation_files is None:
                    print("Error: if --tss_source is False, please assign "
                          "--tex_libs and --annotation_files as well!")
                    sys.exit()
                self.helper.check_make_folder(os.path.join(
                    out_folder, "tmp_wig"))
                self.tex_wigs = self._create_wig_folder(
                        os.path.join(out_folder, "tmp_wig", "tex_notex"),
                        tex_libs)
                tracks = {"file": [], "track": {"+": [], "-": []}}
                self.input_libs = self._create_working_wigs(
                         out_folder, tex_libs, self.tex_wigs, tracks)
                self.libs = self.input_libs
                self = self._parser_combine_wigs("promoter")
        return self

    def container_operon(self, TSS_files, annotation_files,
                         transcript_files, UTR5_files, UTR3_files,
                         term_files, TSS_fuzzy, term_fuzzy, min_length,
                         operon_output_folder, operon_statistics_folder):
        self.tsss = self._gen_copy_new_folder(
                [".gff"], operon_output_folder, "tmp_tss",
                TSS_files, ["--tss_files"])
        self.gffs = self._gen_copy_new_folder(
                [".gff"], operon_output_folder, "tmp_anno",
                annotation_files, ["--annotation_files"])
        self.trans = self._gen_copy_new_folder(
                [".gff"], operon_output_folder, "tmp_ta",
                transcript_files, ["--transcript_files"])
        self.utr5s = self._gen_copy_new_folder(
                [".gff"], operon_output_folder, "tmp_utr5",
                UTR5_files, ["--utr5_files"])
        self.utr3s = self._gen_copy_new_folder(
                [".gff"], operon_output_folder, "tmp_utr3",
                UTR3_files, ["--utr3_files"])
        self.terms = self._gen_copy_new_folder(
                [".gff"], operon_output_folder, "tmp_term",
                term_files, ["--term_files"])
        self.tss_fuzzy = TSS_fuzzy
        self.term_fuzzy = term_fuzzy
        self.length = min_length
        self.output_folder = operon_output_folder
        self.stat_folder = operon_statistics_folder
        return self

    def container_snp(self, samtools_path, bcftools_path, bam_type,
                      program, fasta_files, bam_files,
                      quality, read_depth_range, snp_output_folder,
                      indel_fraction, chrom, rg, caller, filters, DP4_cutoff):
        self.samtools_path = samtools_path
        self.bcftools_path = bcftools_path
        self.types = bam_type
        self.program = program 
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], snp_output_folder,
                "tmp_fa", fasta_files, ["--fasta_files"])
        self.bams = bam_files
        self.quality = quality
        self.depth_s = read_depth_range.split(",")[0]
        self.depth_b = read_depth_range.split(",")[-1]
        self.out_folder = snp_output_folder
        self.idv = indel_fraction.split(",")[0]
        self.imf = indel_fraction.split(",")[-1]
        if chrom == "haploid":
            chrom = "1"
        elif chrom == "diploid":
            chrom = "2"
        self.chrom = chrom
        self.rg = rg
        self.caller = caller
        self.filters = filters
        self.dp4_sum = DP4_cutoff.split(",")[0]
        self.dp4_frac = DP4_cutoff.split(",")[-1]
        return self

    def container_circrna(self, process, fasta_files, annotation_files,
                          bam_files, read_files,
                          circrna_stat_folder, support_reads, segemehl_path,
                          testrealign, samtools_path, start_ratio,
                          end_ratio, ignore_hypothetical_protein, out_folder):
        self.cores = process
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], out_folder, "tmp_fa", fasta_files,
                ["--fasta_files"])
        self.gffs = self._gen_copy_new_folder(
                [".gff"], out_folder, "tmp_anno", annotation_files,
                ["--annotation_files"])
        self.bams = bam_files
        self.read_files = read_files
        self.stat_folder = circrna_stat_folder
        self.support = support_reads
        self.segemehl_path = segemehl_path
        self.testrealign_path = testrealign
        self.samtools_path = samtools_path
        self.start_ratio = start_ratio
        self.end_ratio = end_ratio
        self.hypo = ignore_hypothetical_protein
        self.output_folder = out_folder
        return self

    def container_ribos(self, program, thermo_ID, cmscan_path, cmpress_path,
                        riboswitch_ID, annotation_files, fasta_files,
                        tss_files, transcript_files, Rfam, ribos_output_folder,
                        thermo_output_folder, e_value, output_all,
                        database_folder, fuzzy, start_codon, min_dist_rbs,
                        max_dist_rbs, fuzzy_rbs, UTR_length):
        self.program = program
        if (program.lower() == "riboswitch") or (
                program.lower() == "both"):
            output = ribos_output_folder
        elif (program.lower() == "thermometer"):
            output = thermo_output_folder
        self.thermo_id = thermo_ID
        self.cmscan_path = cmscan_path
        self.cmpress_path = cmpress_path
        self.ribos_id = riboswitch_ID
        self.gffs = self._gen_copy_new_folder(
                [".gff"], output, "temp_anno", annotation_files,
                ["--annotation_files"])
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], output, "temp_fa",
                fasta_files, ["--fasta_files"])
        self.tsss = self._gen_copy_new_folder(
                [".gff"], output, "temp_tss", tss_files, ["--tss_files"])
        self.trans = self._gen_copy_new_folder(
                [".gff"], output, "temp_ta", transcript_files,
                ["--transcript_files"])
        self.rfam = Rfam
        self.ribos_out_folder = ribos_output_folder
        self.thermo_out_folder = thermo_output_folder
        self.e_value = e_value
        self.output_all = output_all
        self.database = database_folder
        self.fuzzy = fuzzy
        self.start_codons = start_codon
        self.start_rbs = min_dist_rbs
        self.end_rbs = max_dist_rbs
        self.fuzzy_rbs = fuzzy_rbs
        self.utr = UTR_length
        return self

    def container_cris(self, fasta_files, annotation_files, CRT_path,
                       window_size, min_number_repeat, min_length_repeat,
                       Max_length_repeat, min_length_spacer, Max_length_spacer,
                       cris_out_folder, ignore_hypo):
        self.gffs = self._gen_copy_new_folder(
                [".gff"], cris_out_folder, "tmp_anno", annotation_files,
                ["--annotation_files"])
        self.fastas = self._gen_copy_new_folder(
                [".fa", ".fna", ".fasta"], cris_out_folder, "tmp_fa",
                fasta_files, ["--fasta_files"])
        self.crt_path = CRT_path
        self.win_size = window_size
        self.out_folder = cris_out_folder
        self.min_num_r = min_number_repeat
        self.min_len_r = min_length_repeat
        self.max_len_r = Max_length_repeat
        self.min_len_s = min_length_spacer
        self.max_len_s = Max_length_spacer
        self.ignore_hypo = ignore_hypo
        return self

    def container_screen(self, main_gff, side_gffs, fasta, height, tex_libs,
                         frag_libs, present, output_folder):
        self.main_gff = main_gff
        self.side_gffs = side_gffs
        self.fasta = fasta
        self.height = height
        self.tlibs = tex_libs
        self.flibs = frag_libs
        self.present = present
        self.output_folder = output_folder
        return self



import os
import sys
import csv
import shutil
from subprocess import call
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from annogesiclib.gff3 import Gff3Parser


class Crispr(object):
    '''Detection of CRISPR'''

    def __init__(self, args_cris):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.gff_parser = Gff3Parser()
        self.gff_path = os.path.join(args_cris.gffs, "tmp")
        self.fasta_path = os.path.join(args_cris.fastas, "tmp")
        self.stat_folder = os.path.join(args_cris.out_folder, "statistics")
        self.gff_out = os.path.join(args_cris.out_folder, "gffs")
        self.all_out = os.path.join(args_cris.out_folder,
                                    "gffs", "all_candidates")
        self.best_out = os.path.join(args_cris.out_folder,
                                     "gffs", "best_candidates")
        self.helper.check_make_folder(self.all_out)
        self.helper.check_make_folder(self.best_out)
        self.data_folder = os.path.join(args_cris.out_folder, "CRT_results")
        self.helper.check_make_folder(self.data_folder)
        self.helper.check_make_folder(self.stat_folder)

    def _run_crt(self, args_cris):
        '''Running CRT'''
        print("Running CRT")
        for seq in os.listdir(self.fasta_path):
            prefix = ".".join(seq.split(".")[:-1])
            call(["java", "-cp", args_cris.crt_path, "crt", "-minNR",
                  str(args_cris.min_num_r), "-minRL",
                  str(args_cris.min_len_r), "-maxRL",
                  str(args_cris.max_len_r), "-minSL",
                  str(args_cris.min_len_s), "-maxSL",
                  str(args_cris.max_len_s), "-searchWL",
                  str(args_cris.win_size), os.path.join(self.fasta_path, seq),
                  os.path.join(self.data_folder, prefix + ".txt")])

    def _read_gff(self, txt):
        gffs = []
        gh = open(os.path.join(self.gff_path,
                               txt.replace(".txt", ".gff")), "r")
        for entry in Gff3Parser().entries(gh):
            if (entry.feature == "gene") or (
                    entry.feature == "CDS") or (
                    entry.feature == "tRNA") or (
                    entry.feature == "rRNA"):
                gffs.append(entry)
        gh.close()
        return gffs

    def _compare_gff(self, strain, start, end, gffs, bh, indexs, ignore_hypo):
        '''Compare CRISPR and genome annotation to 
        remove the false positives'''
        overlap = False
        id_ = None
        for gff in gffs:
            if (gff.seq_id == strain):
                if ((gff.start <= start) and (gff.end >= end)) or (
                        (gff.start >= start) and (gff.end <= end)) or (
                        (gff.start <= start) and (gff.end > start) and (
                            gff.end <= end)) or (
                        (gff.start >= start) and (gff.start < end) and (
                            gff.end >= end)):
                    if "product" in gff.attributes.keys():
                        if ((not ignore_hypo) and ("hypothetical protein" in
                                gff.attributes["product"])) or (
                                "hypothetical protein" not in 
                                gff.attributes["product"]):
                            overlap = True
        if not overlap:
            id_ = "CRISPR_" + str(indexs["best"])
            attribute = ";".join(["ID=" + strain + "_" + id_,
                                  "method=CRT"])
            bh.write("\t".join([strain, "ANNOgesic", "CRISPR", str(start),
                                str(end), ".", ".", ".", attribute]) + "\n")
            indexs["best"] += 1
        return overlap, id_

    def _print_repeat(self, row, strain, file_h, indexs, id_, best):
        '''Print the repeat units'''
        if best:
            num = indexs["re_best"]
        else:
            num = indexs["re_all"]
        if (not row[0].startswith("-")) and (
                not row[0].startswith("Repeats:")) and (
                not row[0].startswith("CRISPR")) and (
                not row[0].startswith("POSITION")):
            start = row[0].strip()
            end = str(int(start) + len(row[2].strip()) - 1)
            attribute = ";".join(["ID=" + strain + "_Repeat_" + str(num),
                                  "method=CRT", "Parent=" + id_])
            file_h.write("\t".join([strain, "ANNOgesic", "repeat_unit",
                                    start, end, ".", ".", ".",
                                    attribute]) + "\n")
            num += 1
        if row[0].startswith("Repeats:"):
            indexs["run"] = False
        return num

    def _convert_gff(self, ignore_hypo):
        '''Convert the final CRT output to gff format'''
        for txt in os.listdir(self.data_folder):
            gffs = self._read_gff(txt)
            fh = open(os.path.join(self.data_folder, txt), "r")
            oh = open(os.path.join(
                self.all_out, txt.replace(".txt", "_CRISPR.gff")), "w")
            bh = open(os.path.join(
                self.best_out, txt.replace(".txt", "_CRISPR.gff")), "w")
            indexs = {"all": 0, "re_all": 0, "best": 0,
                      "re_best": 0, "run": False}
            for row in csv.reader(fh, delimiter='\t'):
                if len(row) != 0:
                    if row[0].startswith("ORGANISM:"):
                        strain = row[0].split(" ")[-1]
                    elif row[0].startswith("CRISPR"):
                        end = row[0].split("-")[-1].strip()
                        start = row[0].split("-")[0].split(":")[-1].strip()
                        id_ = "CRISPR_" + str(indexs["all"])
                        attribute = ";".join(["ID=" + strain + "_" + id_,
                                              "method=CRT"])
                        oh.write("\t".join([
                            strain, "ANNOgesic", "CRISPR", start,
                            end, ".", ".", ".", attribute]) + "\n")
                        overlap, over_id = self._compare_gff(
                                strain, int(start), int(end), gffs, bh,
                                indexs, ignore_hypo)
                        indexs["all"] += 1
                        indexs["run"] = True
                    if indexs["run"]:
                        indexs["re_all"] = self._print_repeat(
                                row, strain, oh, indexs, id_, False)
                        if not overlap:
                            indexs["re_best"] = self._print_repeat(
                                    row, strain, bh, indexs, over_id, True)
            fh.close()
            oh.close()
            bh.close()

    def _stat_and_correct(self, stats, folder):
        '''do statistics and print the final gff file'''
        for gff in os.listdir(folder):
            prefix = gff.replace("_CRISPR.gff", "")
            stats[prefix] = {"all": {"cri": 0, "re": {}}}
            gh = open(os.path.join(folder, gff), "r")
            oh = open("tmp_cri.gff", "w")
            oh.write("##gff-version 3\n")
            cr_num = 0
            re_num = 0
            first = True
            for entry in Gff3Parser().entries(gh):
                if entry.seq_id not in stats[prefix].keys():
                    stats[prefix][entry.seq_id] = {"cri": 0, "re": {}}
                if entry.feature == "CRISPR":
                    id_ = "CRISPR_" + str(cr_num)
                    attribute = ";".join(["ID=" + entry.seq_id + "_" + id_,
                                          "method=CRT"])
                    cr_num += 1
                    if first:
                        first = False
                    else:
                        if repeat not in stats[prefix][entry.seq_id]["re"].keys():
                            stats[prefix][entry.seq_id]["re"][repeat] = 1
                        else:
                            stats[prefix][entry.seq_id]["re"][repeat] += 1
                        if repeat not in stats[prefix]["all"]["re"].keys():
                            stats[prefix]["all"]["re"][repeat] = 1
                        else:
                            stats[prefix]["all"]["re"][repeat] += 1
                    repeat = 0
                    stats[prefix][entry.seq_id]["cri"] += 1
                    stats[prefix]["all"]["cri"] += 1
                elif entry.feature == "repeat_unit":
                    attribute = ";".join(["ID=" + entry.seq_id + "_Repeat_" + str(re_num),
                                          "method=CRT", "Parent=" + id_])
                    re_num += 1
                    repeat += 1
                oh.write("\t".join([entry.info_without_attributes,
                                    attribute]) + "\n")
            if not first:
                if repeat not in stats[prefix][entry.seq_id]["re"].keys():
                    stats[prefix][entry.seq_id]["re"][repeat] = 1
                else:
                    stats[prefix][entry.seq_id]["re"][repeat] += 1
                if repeat not in stats[prefix]["all"]["re"].keys():
                    stats[prefix]["all"]["re"][repeat] = 1
                else:
                    stats[prefix]["all"]["re"][repeat] += 1
            gh.close()
            oh.close()
            os.remove(os.path.join(folder, gff))
            shutil.move("tmp_cri.gff", os.path.join(folder, gff))

    def _print_file(self, sh, cri_res_all, cri_res_best):
        sh.write("\tthe number of CRISPR - {0}\n".format(
                 cri_res_all["cri"]))
        for index, num in cri_res_all["re"].items():
            sh.write("\t\tCRISPR with {0} repeat units - {1}\n".format(
                         index, num))
        sh.write("\tthe number of CRISPR which not overlap "
                 "with genome annotation - {0}\n".format(
                    cri_res_best["cri"]))
        for index, num in cri_res_best["re"].items():
            sh.write("\t\tCRISPR with {0} repeat units - {1}\n".format(
                        index, num))

    def _print_stat(self, stats):
        '''print the statistics file'''
        for prefix, strains in stats["all"].items():
            sh = open(os.path.join(self.stat_folder, prefix + ".csv"), "w")
            if len(strains) == 1:
                sh.write("No CRISPR can be detected")
            elif len(strains) <= 2:
                for strain, cri_res in strains.items():
                    if strain != "all":
                        sh.write(strain + ":\n")
                        self._print_file(sh, cri_res,
                                         stats["best"][prefix][strain])
            else:
                sh.write("All strains:\n")
                self._print_file(sh, stats["all"][prefix]["all"],
                                 stats["best"][prefix]["all"])
                for strain, cri_res in strains.items():
                    if strain != "all":
                        sh.write(strain + ":\n")
                        if strain not in stats["best"][prefix].keys():
                            stats["best"][prefix][strain] = {"cri": 0,
                                                             "re": {}}
                        self._print_file(sh, cri_res,
                                         stats["best"][prefix][strain])
            sh.close()

    def run_crispr(self, args_cris):
        '''detection of CRISPR'''
        self.multiparser.parser_fasta(args_cris.fastas)
        self.multiparser.parser_gff(args_cris.gffs, None)
        self._run_crt(args_cris)
        self._convert_gff(args_cris.ignore_hypo)
        print("All candidates:")
        self.multiparser.combine_gff(args_cris.gffs, self.all_out,
                                     None, "CRISPR")
        print("Best candidates:")
        self.multiparser.combine_gff(args_cris.gffs, self.best_out,
                                     None, "CRISPR")
        stats = {"all": {}, "best": {}}
        self._stat_and_correct(stats["all"], self.all_out)
        self._stat_and_correct(stats["best"], self.best_out)
        self._print_stat(stats)
        self.helper.remove_tmp_dir(args_cris.gffs)
        self.helper.remove_tmp_dir(args_cris.fastas)



import os
import math
import matplotlib as mpl
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper
import numpy as np
mpl.use('Agg')
import matplotlib.pyplot as plt
plt.style.use('ggplot')


def plot(lens, out_figure):
    ticks = max(lens) / 50
    bin_num = np.arange(0, max(lens), ticks)
    n, bins, hist1 = plt.hist(lens, bin_num,
                              color="#FF9999", label='Transcript')
    plt.xlabel("Transcript_length (nt)")
    plt.ylabel("Amount")
    plt.savefig(out_figure)
    plt.clf()


def plot_tran(tran_folder, stat_folder, max_dist):
    lens = []
    less = []
    for tran in os.listdir(tran_folder):
        if tran.endswith(".gff"):
            prefix = tran.replace("_transcript.gff", "")
            gff_f = open(os.path.join(tran_folder, tran), "r")
            for entry in Gff3Parser().entries(gff_f):
                if entry.feature == "transcript":
                    lens.append(entry.end - entry.start)
                    if entry.end - entry.start <= max_dist:
                        less.append(entry.end - entry.start)
            plot(lens, os.path.join(stat_folder, prefix + "_length_all.png"))
            plot(less, os.path.join(stat_folder, prefix + "_length_less_" + 
                                    str(max_dist) + ".png"))




import os
import sys
import shutil
from annogesiclib.helper import Helper
from annogesiclib.multiparser import Multiparser
from annogesiclib.converter import Converter
from annogesiclib.combine_frag_tex import combine
from annogesiclib.stat_TA_comparison import stat_ta_tss, stat_ta_gff
from annogesiclib.transcript_detection import detect_transcript
from annogesiclib.fill_gap import fill_gap, longer_ta
from annogesiclib.gen_table_tran import gen_table_transcript
from annogesiclib.compare_tran_term import compare_term_tran
from annogesiclib.plot_tran import plot_tran


class TranscriptDetection(object):
    '''doing for transcript detection'''

    def __init__(self, args_tran):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.converter = Converter()
        self.gff_outfolder = os.path.join(args_tran.out_folder, "gffs")
        self.tran_path = os.path.join(self.gff_outfolder, "tmp")
        self.stat_path = os.path.join(args_tran.out_folder, "statistics")
        self.tmps = {"gff": "tmp.gff", "merge": "tmp_merge",
                     "tran": os.path.join(args_tran.out_folder, "tmp_tran"),
                     "tss_ta": os.path.join(self.gff_outfolder, "tmp_tss_ta"),
                     "ta_tss": os.path.join(self.gff_outfolder, "tmp_ta_tss"),
                     "ta_gff": os.path.join(self.gff_outfolder, "tmp_ta_gff"),
                     "gff_ta": os.path.join(self.gff_outfolder, "tmp_gff_ta"),
                     "uni": os.path.join(self.gff_outfolder, "tmp_uni"),
                     "overlap": os.path.join(
                         self.gff_outfolder, "tmp_overlap")}
        self.frag = "transcript_fragment.gff"
        self.tex = "transcript_tex_notex.gff"
        self.endfix_tran = "transcript.gff"

    def _compute_transcript(self, wig_f, wig_r, wig_folder, wig_type, strain,
                            libs, args_tran):
        print("Computing transcripts for {0}".format(strain))
        out = os.path.join(args_tran.out_folder, "_".join([strain, wig_type]))
        detect_transcript(wig_f, wig_r, wig_folder, libs, out, wig_type, args_tran)

    def _compute(self, wig_type, wigs, libs, args_tran):
        strains = []
        wig_folder = os.path.join(wigs, "tmp")
        for wig in os.listdir(wig_folder):
            if wig.endswith("_forward.wig"):
                strains.append(wig.replace("_forward.wig", ""))
        for strain in strains:
            f_file = os.path.join(wig_folder, "_".join(
                [strain, "forward.wig"]))
            r_file = os.path.join(wig_folder, "_".join(
                [strain, "reverse.wig"]))
            self._compute_transcript(f_file, r_file, wigs, wig_type,
                                     strain, libs, args_tran)
        return strains

    def _compare_tss(self, tas, args_tran):
        self.multiparser.parser_gff(args_tran.compare_tss, "TSS")
        self.multiparser.combine_gff(
                self.gff_outfolder,
                os.path.join(args_tran.compare_tss, "tmp"),
                "transcript", "TSS")
        print("Comaring of transcripts and TSSs")
        tss_folder = os.path.join(args_tran.compare_tss, "tmp")
        for ta in tas:
            ta_file = os.path.join(self.gff_outfolder,
                                   "_".join([ta, self.endfix_tran]))
            stat_tss_out = os.path.join(
                    self.stat_path, "".join([
                        "stat_compare_transcript_TSS_",
                        ta, ".csv"]))
            for tss in os.listdir(tss_folder):
                filename = tss.split("_TSS")
                if (filename[0] == ta) and (tss.endswith(".gff")):
                    stat_ta_tss(ta_file, os.path.join(tss_folder, tss),
                                stat_tss_out, self.tmps["ta_tss"],
                                self.tmps["tss_ta"], args_tran.fuzzy)
                    os.remove(ta_file)
                    os.remove(os.path.join(tss_folder, tss))
                    self.helper.sort_gff(self.tmps["ta_tss"], ta_file)
                    self.helper.sort_gff(
                            self.tmps["tss_ta"], os.path.join(
                                args_tran.compare_tss, tss))
                    os.remove(self.tmps["tss_ta"])
                    os.remove(self.tmps["ta_tss"])

    def _compare_cds(self, tas, args_tran):
        self.multiparser.parser_gff(args_tran.gffs, None)
        self.multiparser.combine_gff(
            self.gff_outfolder, os.path.join(args_tran.gffs, "tmp"),
            "transcript", None)
        print("Comaring of transcripts and genome annotations")
        cds_folder = os.path.join(args_tran.gffs, "tmp")
        for ta in tas:
            ta_file = os.path.join(self.gff_outfolder,
                                   "_".join([ta, self.endfix_tran]))
            stat_gff_out = os.path.join(self.stat_path, "".join([
                "stat_compare_transcript_genome_", ta, ".csv"]))
            for gff in os.listdir(cds_folder):
                if (gff[:-4] == ta) and (gff.endswith(".gff")):
                    cds_file = os.path.join(cds_folder, gff)
                    stat_ta_gff(ta_file, cds_file, stat_gff_out,
                                self.tmps["ta_gff"], self.tmps["gff_ta"],
                                args_tran.c_feature)
                    os.remove(ta_file)
                    os.remove(os.path.join(args_tran.gffs, gff))
                    self.helper.sort_gff(self.tmps["ta_gff"], ta_file)
                    self.helper.sort_gff(self.tmps["gff_ta"], os.path.join(
                        args_tran.gffs, gff))
                    os.remove(self.tmps["ta_gff"])
                    os.remove(self.tmps["gff_ta"])

    def _compare_tss_cds(self, tas, args_tran):
        '''compare transcript with CDS and TSS'''
        if (args_tran.compare_tss is not None) and (
                args_tran.c_feature is not None):
            self.multiparser.parser_gff(self.gff_outfolder, "transcript")
            self._compare_cds(tas, args_tran)
            self._compare_tss(tas, args_tran)
        elif (args_tran.c_feature is not None) and (
                args_tran.compare_tss is None):
            self.multiparser.parser_gff(self.gff_outfolder, "transcript")
            self._compare_cds(tas, args_tran)
        elif (args_tran.c_feature is None) and (
                args_tran.compare_tss is not None):
            self.multiparser.parser_gff(self.gff_outfolder, "transcript")
            self._compare_tss(tas, args_tran)

    def _for_one_wig(self, type_, args_tran):
        '''running transcript detection to one type of wig files'''
        if type_ == "tex_notex":
            libs = args_tran.tlibs
            wigs = args_tran.tex_wigs
        else:
            libs = args_tran.flibs
            wigs = args_tran.frag_wigs
        print("Importing {0} wig files".format(type_))
        strains = self._compute(type_, wigs, libs, args_tran)
        for strain in strains:
            out = os.path.join(self.gff_outfolder, "_".join([
                strain, "transcript", type_ + ".gff"]))
            self.helper.sort_gff(os.path.join(args_tran.out_folder,
                                 "_".join([strain, type_])), out)
            os.remove(os.path.join(args_tran.out_folder,
                                   "_".join([strain, type_])))
        return strains

    def _for_two_wigs(self, strains, args_tran):
        '''merge the results of fragemented and tex treated libs'''
        if (args_tran.frag_wigs is not None) and (
                args_tran.tex_wigs is not None):
            print("Merging fragmented and tex treated ones")
            for strain in strains:
                frag_gff = os.path.join(self.gff_outfolder,
                                        "_".join([strain, self.frag]))
                tex_gff = os.path.join(self.gff_outfolder,
                                       "_".join([strain, self.tex]))
                final_gff = os.path.join(self.gff_outfolder,
                                         "_".join([strain, self.endfix_tran]))
                for gff in os.listdir(self.gff_outfolder):
                    if "_transcript_" in gff:
                        filename = gff.split("_transcript_")
                        if (strain == filename[0]) and (
                                "tex_notex.gff" == filename[1]):
                            tex_file = gff
                        elif (strain == filename[0]) and (
                                "fragment.gff" == filename[1]):
                            frag_file = gff
                combine(os.path.join(self.gff_outfolder, frag_file),
                        os.path.join(self.gff_outfolder, tex_file),
                        args_tran.tolerance,
                        os.path.join(self.gff_outfolder,
                                     "_".join([strain, self.endfix_tran])))
                os.remove(frag_gff)
                os.remove(tex_gff)
        else:
            if args_tran.frag_wigs is not None:
                for strain in strains:
                    frag_gff = os.path.join(
                            self.gff_outfolder, "_".join([strain, self.frag]))
                    final_gff = os.path.join(
                            self.gff_outfolder,
                            "_".join([strain, self.endfix_tran]))
                    shutil.move(frag_gff, final_gff)
            elif args_tran.tex_wigs is not None:
                for strain in strains:
                    tex_gff = os.path.join(
                            self.gff_outfolder, "_".join([strain, self.tex]))
                    final_gff = os.path.join(
                            self.gff_outfolder,
                            "_".join([strain, self.endfix_tran]))
                    shutil.move(tex_gff, final_gff)

    def _post_modify(self, tas, args_tran):
        '''modify the transcript by comparing with genome annotation'''
        for ta in tas:
            for gff in os.listdir(args_tran.gffs):
                if (".gff" in gff) and (gff[:-4] == ta):
                    break
            print("Modifying {0} by refering to {1}".format(ta, gff))
            fill_gap(os.path.join(args_tran.gffs, gff),
                     os.path.join(self.tran_path,
                     "_".join([ta, self.endfix_tran])),
                     "overlap", self.tmps["overlap"], args_tran.modify)
            fill_gap(os.path.join(args_tran.gffs, gff),
                     os.path.join(self.tran_path,
                     "_".join([ta, self.endfix_tran])),
                     "uni", self.tmps["uni"], args_tran.modify)
            tmp_merge = os.path.join(self.gff_outfolder, self.tmps["merge"])
            if self.tmps["merge"] in self.gff_outfolder:
                os.remove(tmp_merge)
            self.helper.merge_file(self.tmps["overlap"], tmp_merge)
            self.helper.merge_file(self.tmps["uni"], tmp_merge)
            tmp_out = os.path.join(self.gff_outfolder, "_".join(["tmp", ta]))
            self.helper.sort_gff(tmp_merge, tmp_out)
            os.remove(self.tmps["overlap"])
            os.remove(self.tmps["uni"])
            os.remove(tmp_merge)
            final_out = os.path.join(self.gff_outfolder,
                                     "_".join(["final", ta]))
            longer_ta(tmp_out, args_tran.length, final_out)
            shutil.move(final_out,
                        os.path.join(self.tmps["tran"],
                                     "_".join([ta, self.endfix_tran])))
            os.remove(tmp_out)
        shutil.rmtree(self.gff_outfolder)
        shutil.move(self.tmps["tran"], self.gff_outfolder)

    def _remove_file(self, args_tran):
        if "tmp_wig" in os.listdir(args_tran.out_folder):
            shutil.rmtree(os.path.join(args_tran.out_folder, "tmp_wig"))
        self.helper.remove_tmp_dir(args_tran.gffs)
        self.helper.remove_tmp_dir(args_tran.compare_tss)
        self.helper.remove_tmp_dir(args_tran.terms)
        self.helper.remove_tmp(os.path.join(args_tran.out_folder, "gffs"))
        self.helper.remove_tmp(self.gff_outfolder)

    def _compare_term_tran(self, args_tran):
        '''searching the associated terminator to transcript'''
        if args_tran.terms is not None:
            print("Comparing between terminators and transcripts")
            self.multiparser.parser_gff(args_tran.terms, "term")
            if args_tran.gffs is not None:
                self.multiparser.combine_gff(
                    args_tran.gffs,
                    os.path.join(args_tran.terms, "tmp"), None, "term")
            compare_term_tran(self.gff_outfolder,
                              os.path.join(args_tran.terms, "tmp"),
                              args_tran.fuzzy_term, args_tran.fuzzy_term,
                              args_tran.out_folder, "transcript",
                              args_tran.terms, self.gff_outfolder)

    def run_transcript(self, args_tran):
        if (args_tran.frag_wigs is None) and (args_tran.tex_wigs is None):
            print("Error: There is no wigs file!\n")
            sys.exit()
        if args_tran.frag_wigs is not None:
            strains = self._for_one_wig("fragment", args_tran)
        if args_tran.tex_wigs is not None:
            strains = self._for_one_wig("tex_notex", args_tran)
        self._for_two_wigs(strains, args_tran)
        tas = []
        if "none" not in args_tran.modify:
            for gff in os.listdir(args_tran.gffs):
                if gff.endswith(".gff"):
                    self.helper.sort_gff(os.path.join(args_tran.gffs, gff),
                                         self.tmps["gff"])
                    shutil.move(self.tmps["gff"],
                                os.path.join(args_tran.gffs, gff))
            self.multiparser.combine_gff(args_tran.gffs, os.path.join(
                args_tran.gffs, "tmp"), None, None)
            self.multiparser.parser_gff(self.gff_outfolder, "transcript")
            self.multiparser.combine_gff(args_tran.gffs, self.tran_path,
                                         None, "transcript")
            self.helper.check_make_folder(self.tmps["tran"])
            for ta in os.listdir(self.tran_path):
                if ta.endswith(".gff"):
                    if os.path.getsize(os.path.join(self.tran_path, ta)) != 0:
                        tas.append(ta.replace("_" + self.endfix_tran, ""))
            self._post_modify(tas, args_tran)
        self._compare_tss_cds(tas, args_tran)
        self._compare_term_tran(args_tran)
        print("Generating tables for the details")
        gen_table_transcript(self.gff_outfolder, args_tran)
        plot_tran(self.gff_outfolder, self.stat_path, args_tran.max_dist)
        self._remove_file(args_tran)



from annogesiclib.gff3 import Gff3Parser


def uni(tas, genes, out):
    '''This is for the transcript which is not overlap with annotation'''
    start_tmp = 0
    stop_tmp = 0
    strand_tmp = ""
    detect = False
    for ta in tas:
        for gene in genes:
            if (ta.strand == gene.strand) and (
                    ta.seq_id == gene.seq_id):
                if ((ta.start < gene.start) and (
                         ta.end > gene.start) and (
                         ta.end < gene.end)) or (
                        (ta.start > gene.start) and (
                         ta.end < gene.end)) or (
                        (ta.start > gene.start) and (
                         ta.start < gene.end) and (
                         ta.end > gene.end)):
                    detect = True
        if ((not detect) and (start_tmp != ta.start) and (
                stop_tmp != ta.end)) or (
                (not detect) and (((start_tmp == ta.start) or (
                    stop_tmp == ta.end)) and (
                    strand_tmp != ta.strand))):
            out.write(ta.info + "\n")
            start_tmp = ta.start
            stop_tmp = ta.end
            strand_tmp = ta.strand
        detect = False


def check_modify(start_tmp, stop_tmp, gene, ta, modify):
    if (ta.start >= gene.start) and (
            ta.end <= gene.end):
        if "within_extend_ends" in modify:
            start_tmp = gene.start
            stop_tmp = gene.end
        else:
            start_tmp = ta.start
            stop_tmp = ta.end
    elif ((ta.start <= gene.start) and (
            ta.end >= gene.start) and (
            ta.end <= gene.end)):
        if (ta.strand == "+") and ("extend_3end" in modify):
            start_tmp = ta.start
            stop_tmp = gene.end
        elif (ta.strand == "-") and ("extend_5end" in modify):
            start_tmp = ta.start
            stop_tmp = gene.end
        else:
            start_tmp = ta.start
            stop_tmp = ta.end
    elif ((ta.start >= gene.start) and (
            ta.start <= gene.end) and (
            ta.end >= gene.end)):
        if (ta.strand == "+") and ("extend_5end" in modify):
            start_tmp = gene.start
            stop_tmp = ta.end
        elif (ta.strand == "-") and ("extend_3end" in modify):
            start_tmp = gene.start
            stop_tmp = ta.end
        else:
            start_tmp = ta.start
            stop_tmp = ta.end
    return start_tmp, stop_tmp


def overlap(tas, genes, out, modify):
    '''Check the overlap of annotation and transcript'''
    check = False
    for gene in genes:
        start_tmp = 0
        stop_tmp = 0
        start = 0
        stop = 0
        for ta in tas:
            if (ta.strand == gene.strand) and (
                    ta.seq_id == gene.seq_id):
                if ((ta.start <= gene.start) and (
                         ta.end >= gene.start) and (
                         ta.end <= gene.end)) or (
                        (ta.start >= gene.start) and (
                         ta.end <= gene.end)) or (
                        (ta.start >= gene.start) and (
                         ta.start <= gene.end) and (
                         ta.end >= gene.end)):
                    check = True
                    tmp_ta = ta
                    if start_tmp == 0:
                        start_tmp, stop_tmp = check_modify(
                            start_tmp, stop_tmp, gene, ta, modify)
                        start = start_tmp
                        stop = stop_tmp
                    else:
                        start_tmp, stop_tmp = check_modify(
                            start_tmp, stop_tmp, gene, ta, modify)
                        if "merge_overlap" in modify:
                            if stop < stop_tmp:
                                stop = stop_tmp
                        else:
                            if (start_tmp != 0):
                                out.write("\t".join([str(field) for field in [
                                    tmp_ta.seq_id, tmp_ta.source,
                                    tmp_ta.feature,
                                    start, stop, tmp_ta.score,
                                    tmp_ta.strand, tmp_ta.phase,
                                    tmp_ta.attribute_string]]) + "\n")
                                start = start_tmp
                                stop = stop_tmp
                if (ta.start > gene.end) and (start != 0) and (check):
                    check = False
                    out.write("\t".join([str(field) for field in [
                              tmp_ta.seq_id, tmp_ta.source, tmp_ta.feature,
                              start, stop, tmp_ta.score,
                              tmp_ta.strand, tmp_ta.phase,
                              tmp_ta.attribute_string]]) + "\n")
                    break
        if (start != 0) and (check):
            out.write('\t'.join([str(field) for field in [
                      tmp_ta.seq_id, tmp_ta.source, tmp_ta.feature,
                      start, stop, tmp_ta.score, tmp_ta.strand,
                      tmp_ta.phase, tmp_ta.attribute_string]]) + "\n")


def fill_gap(gff_file, ta_file, type_, output, modify):
    '''compare transcript with genome annotation to modify the transcript'''
    tas = []
    genes = []
    ta_f = open(ta_file, "r")
    gff_f = open(gff_file, "r")
    for entry in Gff3Parser().entries(ta_f):
        tas.append(entry)
    ta_f.close()
    tas = sorted(tas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    for entry in Gff3Parser().entries(gff_f):
        if (entry.feature == "gene") or (
                entry.feature == "CDS") or (
                entry.feature == "rRNA") or (
                entry.feature == "tRNA"):
            genes.append(entry)
    gff_f.close()
    genes = sorted(genes, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    out = open(output, "w")
    out.write("##gff-version 3\n")
    if type_ == "overlap":
        overlap(tas, genes, out, modify)
    elif type_ == "uni":
        uni(tas, genes, out)


def print_file(ta, num, out):
    ta.attributes["ID"] = ta.seq_id + "_transcript" + str(num)
    ta.attributes["Name"] = "transcript_" + ('%0*d' % (5, num))
    attribute_string = ";".join(
        ["=".join(items) for items in ta.attributes.items()])
    out.write("\t".join([str(field) for field in [
              ta.seq_id, ta.source, ta.feature, ta.start,
              ta.end, ta.score, ta.strand, ta.phase,
              attribute_string]]) + "\n")


def longer_ta(ta_file, length, out_file):
    '''merge overlaped transcript to for a complete transcript'''
    tas = []
    for entry in Gff3Parser().entries(open(ta_file)):
        tas.append(entry)
    tas = sorted(tas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    for ta_1 in tas:
        for ta_2 in tas:
            if (ta_1.seq_id == ta_2.seq_id) and (
                    ta_1.strand == ta_2.strand):
                if (ta_1.start <= ta_2.start) and (
                        ta_1.end >= ta_2.start) and (
                        ta_1.end <= ta_2.end):
                    ta_1.end = ta_2.end
                elif (ta_1.start >= ta_2.start) and (
                        ta_1.start <= ta_2.end) and (
                        ta_1.end >= ta_2.end):
                    ta_1.start = ta_2.start
                elif (ta_1.start <= ta_2.start) and (
                        ta_1.end >= ta_2.end):
                    pass
                elif (ta_1.start >= ta_2.start) and (
                        ta_1.end <= ta_2.end):
                    ta_1.start = ta_2.start
                    ta_1.end = ta_2.end
    first = True
    out = open(out_file, "w")
    out.write("##gff-version 3\n")
    num = 0
    pre_ta = None
    tas = sorted(tas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    for ta in tas:
        if (ta.end - ta.start) >= length:
            if first:
                first = False
                print_file(ta, num, out)
                num += 1
            else:
                if (ta.seq_id == pre_ta.seq_id) and (
                        ta.strand == pre_ta.strand) and (
                        ta.start == pre_ta.start) and (
                        ta.end == pre_ta.end):
                    pass
                else:
                    print_file(ta, num, out)
                    num += 1
        pre_ta = ta
    out.close()



import os
import sys
from annogesiclib.gen_screenshots import gen_screenshot
from annogesiclib.helper import Helper


class Screen(object):
    '''generation of screenshot'''

    def __init__(self, args_sc):
        self.helper = Helper()
        out_folder = os.path.join(args_sc.output_folder, "screenshots")
        if os.path.exists(out_folder):
            print("Error: The {0} already exists!".format(
                  out_folder))
            sys.exit()
        else:
            os.mkdir(out_folder)
        args_sc.output_folder = out_folder
        filename = args_sc.fasta.split("/")[-1]
        self.strain = ".".join(filename.split(".")[0:-1])
        self.helper.check_make_folder(os.path.join(args_sc.output_folder,
                                                   self.strain))
        self.forward_file = os.path.join(args_sc.output_folder,
                                         self.strain, "forward")
        self.reverse_file = os.path.join(args_sc.output_folder,
                                         self.strain, "reverse")
        os.mkdir(self.forward_file)
        os.mkdir(self.reverse_file)

    def _import_libs(self, texs, strand, lib_dict):
        if strand == "+":
            tex = "ft"
            notex = "fn"
        else:
            tex = "rt"
            notex = "rn"
        for flib in texs:
            if (flib[1] == "tex"):
                lib_dict[tex].append(flib[0])
                for nlib in texs:
                    if (nlib[1] == "notex") and \
                       (flib[2] == nlib[2]) and \
                       (flib[3] == nlib[3]):
                        lib_dict[notex].append(nlib[0])

    def screenshot(self, args_sc):
        lib_dict = {"ft": [], "fn": [], "rt": [], "rn": [], "ff": [], "rf": []}
        f_texs = []
        r_texs = []
        if args_sc.tlibs is not None:
            for lib in args_sc.tlibs:
                lib_datas = lib.split(":")
                if not lib_datas[0].endswith(".wig"):
                    print("Error: Wiggle files should end with .wig!")
                    sys.exit()
                else:
                    if lib_datas[-1] == "+":
                        f_texs.append(lib_datas)
                    else:
                        r_texs.append(lib_datas)
            f_texs = sorted(f_texs, key=lambda x: (x[1], x[2], x[3]))
            r_texs = sorted(r_texs, key=lambda x: (x[1], x[2], x[3]))
            self._import_libs(f_texs, "+", lib_dict)
            self._import_libs(r_texs, "-", lib_dict)
        if args_sc.flibs is not None:
            for lib in args_sc.flibs:
                lib_datas = lib.split(":")
                if not lib_datas[0].endswith(".wig"):
                    print("Error: Wiggle files should end with .wig!")
                    sys.exit()
                else:
                    if lib_datas[-1] == "+":
                        lib_dict["ff"].append(lib_datas[0])
                    else:
                        lib_dict["rf"].append(lib_datas[0])
        gen_screenshot(args_sc, lib_dict, self.forward_file + ".txt",
                       self.reverse_file + ".txt", self.strain)
        if (args_sc.tlibs is None) and (args_sc.flibs is None):
            print("Error: There is no wig file assigned!")
            sys.exit()



import os
import csv
from annogesiclib.gff3 import Gff3Parser
import numpy as np
import copy
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
plt.style.use('ggplot')


def import_uniprot_data(entry, name_list, feature):
    '''import uniprot to dict'''
    ref_name = entry.attributes[feature]
    if ref_name not in name_list:
        name_list.add(ref_name)


def compare_cds_tran(gffs, trans):
    '''compare CDS with transcript to get the expressed CDS'''
    new_gffs = []
    for gff in gffs:
        for ta in trans:
            if (gff.seq_id == ta.seq_id) and (
                    gff.strand == ta.strand):
                if ((gff.end < ta.end) and (
                         gff.end > ta.start) and (
                         gff.start <= ta.start)) or (
                        (gff.start > ta.start) and (
                         gff.start < ta.end) and (
                         gff.end >= ta.end)) or (
                        (gff.end >= ta.end) and (
                         gff.start <= ta.start)) or (
                        (gff.end <= ta.end) and (
                         gff.start >= ta.start)):
                    new_gffs.append(gff)
                    break
    return new_gffs


def get_go_id(gffs, id_, uni_lines, gos):
    '''get the GO id of CDS'''
    detect = False
    for gff in gffs:
        if ("Name" in gff.attributes.keys()):
            if (id_ == gff.attributes["Name"]):
                detect = True
        if ("protein_id" in gff.attributes.keys()):
            if (id_ == gff.attributes["protein_id"]):
                detect = True
        if detect:
            detect = False
            gos.append({"strain": gff.seq_id, "strand": gff.strand,
                        "start": gff.start, "end": gff.end,
                        "protein_id": id_, "go": uni_lines[6]})
            gff.attributes["print"] = True


def print_go(gos, out):
    gos = sorted(gos, key=lambda x: (x["strain"], x["start"],
                                     x["end"], x["strand"]))
    pre_go = None
    for go in gos:
        if (go != pre_go) and (pre_go is not None):
            if (go["strain"] == pre_go["strain"]) and (
                    go["strand"] == pre_go["strand"]) and (
                    go["start"] == pre_go["start"]) and (
                    go["end"] == pre_go["end"]) and (
                    go["protein_id"] == pre_go["protein_id"]):
                go_ids = []
                for go_id in go["go"].split("; "):
                    if go_id not in go_ids:
                        go_ids.append(go_id)
                for go_id in pre_go["go"].split("; "):
                    if go_id not in go_ids:
                        go_ids.append(go_id)
                pre_go["go"] = "; ".join(go_ids)
                out.write("\t".join([pre_go["strain"], pre_go["strand"],
                          str(pre_go["start"]), str(pre_go["end"]),
                          pre_go["protein_id"], pre_go["go"]]) + "\n")
                pre_go["print"] = True
                go["print"] = True
            else:
                if "print" not in pre_go.keys():
                    out.write("\t".join([pre_go["strain"], pre_go["strand"],
                              str(pre_go["start"]), str(pre_go["end"]),
                              pre_go["protein_id"], pre_go["go"]]) + "\n")
                    pre_go["print"] = True
        pre_go = copy.deepcopy(go)
    if "print" not in pre_go.keys():
        out.write("\t".join([pre_go["strain"], pre_go["strand"],
                  str(pre_go["start"]), str(pre_go["end"]),
                  pre_go["protein_id"], pre_go["go"]]) + "\n")

def retrieve_uniprot(database_file, gff_file, out_file, tran_file, type_):
    '''Retrieve the GO term from Uniprot'''
    name_list = set()
    gffs = []
    out = open(out_file, "w")
    out.write("\t".join(["Genome", "Strand", "Start", "End",
                         "Protein_id", "Go_term"]) + "\n")
    for entry in Gff3Parser().entries(open(gff_file)):
        if entry.feature == "CDS":
            if ("Name" in entry.attributes.keys()) and (
                    "protein_id" in entry.attributes.keys()):
                if entry.attributes["Name"] == entry.attributes["protein_id"]:
                    import_uniprot_data(entry, name_list, "Name")
                else:
                    import_uniprot_data(entry, name_list, "Name")
                    import_uniprot_data(entry, name_list, "protein_id")
            elif ("Name" in entry.attributes.keys()):
                import_uniprot_data(entry, name_list, "Name")
            elif ("protein_id" in entry.attributes.keys()):
                import_uniprot_data(entry, name_list, "protein_id")
            gffs.append(entry)
    if (type_ == "express") and (tran_file is not None):
        trans = []
        for entry in Gff3Parser().entries(open(tran_file)):
            trans.append(entry)
        new_gffs = compare_cds_tran(gffs, trans)
        gffs = new_gffs
    idmapping = open(database_file, "r")
    gos = []
    for uni_id in idmapping:
        uni_line = uni_id.rstrip("\n")
        uni_lines = uni_line.split("\t")
        uni_ids = uni_lines[3].split(";")
        for id_ in uni_ids:
            id_ = id_.strip()
            if id_ in name_list:
                get_go_id(gffs, id_, uni_lines, gos)
    for gff in gffs:
        if "print" not in gff.attributes.keys():
            gos.append({"strain": gff.seq_id, "strand": gff.strand,
                        "start": gff.start, "end": gff.end,
                        "protein_id": id_, "go": ""})
    print_go(gos, out)
    out.close()
    idmapping.close()


def plot(total_nums, strain, filename, total, out_folder):
    '''plot the distribution of GO term by GOslim'''
    sort_total_nums = sorted(total_nums.items(),
                             key=lambda x: (x[1]), reverse=True)
    classes = []
    nums = []
    width = 0.4
    plt.figure(figsize=(16, 12))
    for total_num in sort_total_nums:
        class_ = total_num[0]
        num = total_num[1]
        if class_ != "total":
            percent = (float(num) / float(total)) * 100
#            if percent >= 3:
            classes.append(class_.replace("_", " "))
            nums.append(num)
    ind = np.arange(len(nums))
    plt.bar(ind, nums, width, color='#FF9999')
    if filename == "three_roots":
        title = "Distribution of GO term hits in the three root classes"
    elif (filename == "molecular_function") or (
            filename == "cellular_component") or (
            filename == "biological_process"):
        tag = filename.replace("_", " ")
        title = " -- ".join(["Distribution of GO term of the class", tag])
    plt.title(title, fontsize=22)
    plt.ylabel('Amount', fontsize=16)
    plt.xlim([0, len(nums) + 1])
    plt.yticks(fontsize=16)
    plt.xticks(ind+width, classes, rotation=45, fontsize=16, ha='right')
    plt.tight_layout(3, None, None, None)
    plt.savefig(os.path.join(out_folder,
                             "_".join([strain, filename + ".png"])))


def import_obo(filename):
    '''import the information of obo file to dict'''
    obos = []
    start = False
    with open(filename, "r") as o_h:
        for line in o_h:
            line = line.strip()
            if line == "[Term]":
                obo = {}
                start = True
            elif start:
                if len(line) == 0:
                    obos.append(obo.copy())
                    start = False
                else:
                    datas = line.split(": ")
                    if datas[0] == "is_a":
                        if "is_a" not in obo.keys():
                            obo["is_a"] = []
                        obo["is_a"].append(datas[1].strip())
                    else:
                        obo[datas[0]] = datas[1].strip()
    return obos


def import_class(slim_obo, classes, strain):
    if slim_obo["name"] not in classes[strain][slim_obo["namespace"]]:
        classes[strain][slim_obo["namespace"]][slim_obo["name"]] = 0
    classes[strain][slim_obo["namespace"]][slim_obo["name"]] += 1


def import_total(slim_obo, total_nums, strain):
    total_nums[strain][slim_obo["namespace"]] += 1
    total_nums[strain]["total"] += 1


def print_file(classes, total_nums, out_folder, stat):
    out_stat = open(stat, "w")
    printed = True
    for strain, datas in classes.items():
        if (strain == "All_genome") and len(classes) <= 2:
            printed = False
        if printed:
            plot(total_nums[strain], strain, "three_roots",
                 total_nums[strain]["total"], out_folder)
            out_stat.write("{0}:\n".format(strain))
            for origin, types in datas.items():
                plot(types, strain, origin,
                     total_nums[strain][origin], out_folder)
                out_stat.write("\t{0}: {1}(percentage in total: {2})\n".format(
                               origin, total_nums[strain][origin],
                               float(total_nums[strain][origin]) /
                               float(total_nums[strain]["total"])))
                for type_, num in types.items():
                    out_stat.write("\t\t{0}: {1}(percentage "
                                   "in {2}: {3})\n".format(
                                       type_, num, origin, float(num) /
                                       float(total_nums[strain][origin])))
        else:
            printed = True
    out_stat.close()


def initiate_dict(classes, total_nums, index):
    classes[index] = {"biological_process": {},
                      "cellular_component": {},
                      "molecular_function": {}}
    total_nums[index] = {"biological_process": 0,
                         "cellular_component": 0,
                         "molecular_function": 0,
                         "total": 0}


def compare_go_slim(gos, term_obos, slim_obos, classes, total_nums):
    '''Compare GO term and GOslim database'''
    detect = False
    for strain, pros in gos.items():
        for pro, go_ids in pros.items():
            pro_list = []
            for go_id in go_ids:
                target_terms = [go_id]
                for target_term in target_terms:
                    for term_obo in term_obos:
                        if target_term == term_obo["id"]:
                            if "is_a" in term_obo.keys():
                                for is_a in term_obo["is_a"]:
                                    go_a = is_a.split(" ! ")
                                    if (go_a[1] != "biological_process") and (
                                        go_a[1] != "cellular_component") and (
                                            go_a[1] != "molecular_function"):
                                        target_terms.append(go_a[0])
                            elif ("is_obsolete" in term_obo.keys()):
                                if term_obo["is_obsolete"] == "true":
                                    break
                            for slim_obo in slim_obos:
                                for target_term in target_terms:
                                    if (target_term == slim_obo["id"]) and (
                                            target_term not in pro_list):
                                        detect = True
                                        import_class(slim_obo, classes, strain)
                                        import_class(slim_obo, classes,
                                                     "All_genome")
                                        import_total(slim_obo, total_nums,
                                                     strain)
                                        import_total(slim_obo, total_nums,
                                                     "All_genome")
                                        pro_list.append(target_term)
                            break
                    if detect:
                        detect = False
                        break


def map2goslim(slim_file, term_file, go_table, stat, out_folder):
    '''For mapping the GO to GOslim'''
    gos = {}
    classes = {}
    total_nums = {}
    initiate_dict(classes, total_nums, "All_genome")
    pre_strain = ""
    g_h = open(go_table, "r")
    print("Loading go table")
    for row in csv.reader(g_h, delimiter="\t"):
        if row[0] != "Genome":
            if row[0] != pre_strain:
                gos[row[0]] = {}
                initiate_dict(classes, total_nums, row[0])
            go_terms = row[-1].split("; ")
            gos[row[0]]["\t".join(row[1:4])] = go_terms
            pre_strain = row[0]
    print("Loading obo file")
    term_obos = import_obo(term_file)
    slim_obos = import_obo(slim_file)
    print("Starting mapping")
    compare_go_slim(gos, term_obos, slim_obos, classes, total_nums)
    print("Doing statistics and ploting")
    print_file(classes, total_nums, out_folder, stat)
    g_h.close()
